#!/usr/bin/perl -w

=head1 NAME

B<Regexp::Common::RealHTML> - 

=head1 SYNOPSIS

 use Regexp::Common qw/ RealHTML /;

=head1 DESCRIPTION

  "REAL HTML"  "HTML 4.0":
 -        '', "", ``   .
 -        [^'"`\s>=].
 -        [^'"`\s>].
 -            =.
 -     : <TAG =value>.
 -  :
   *  ``<!--''   ``-->'',  ``--''  
   *  : ``<!-->''  ``<!--->''
   *  ``-->''  ,   ``<!--''   ``>''
   *  ``<?''   ``?>''
   *  : ``<?>''

=cut

package Regexp::Common::RealHTML;

#use strict;
#no strict 'subs'; # Bareword "Apache" not allowed: "$RE{x}{-word=>Apache}"

our $VERSION = "1.02";

use Regexp::Common qw /pattern clean no_defaults/;

#    :  ``TYPE''  ``"TYPE"''
my $pname   = qr/ (?:  (?<=\s)  [\"\'\`]?  (
		    (?<=\s)  [^\"\'\`\s>=]+  (?=[\s>=])	|
		    (?<=\")  [^\"]+	     (?=\")	|
		    (?<=\')  [^\']+	     (?=\')	|
		    (?<=\`)  [^\`]+	     (?=\`)
		    )  [\"\'\`]?  ) /x;

#    : ``VALUE''  ``"VALUE"''
my $pval    = qr/ (?: (?<=\=)  [\"\'\`]?  (
		    (?<=\=)  [^\"\'\`\s>]*   (?=[\s>])	|
		    (?<=\")  [^\"]*	     (?=\")	|
		    (?<=\')  [^\']*	     (?=\')	|
		    (?<=\`)  [^\`]*	     (?=\`)
		    )  [\"\'\`]? ) /x;

#   : `` =''  `` TYPE=''  `` =VALUE''  `` TYPE=VALUE''
#  `` TYPE''
my $param   = qr/ (?: \s+ $pname? (?: = $pval? )? ) /xo;

#      qr//: ``Sequence (?k...) not recognized in regex''
s/\((?=\s)/(?k:/g for $pname, $pval, $param;
pattern name => [ 'pname'   ], create => $pname;
pattern name => [ 'pval'    ], create => $pval;
pattern name => [ 'param'   ], create => $param;

#    <>
my $tag	    = qr/ (?: [!\/]? \w+ $RE{param}* \s* ) /xo;

#      <>: ``TITLE>....</TITLE''
my $ctag    = qr# (?:
    TITLE  $RE{param}* \s* > [^<]* (?:<(?!/TITLE) [^<]*)* </TITLE  $RE{param}* \s* |
    STYLE  $RE{param}* \s* > [^<]* (?:<(?!/STYLE) [^<]*)* </STYLE  $RE{param}* \s* |
    SCRIPT $RE{param}* \s* > [^<]* (?:<(?!/SCRIPT)[^<]*)* </SCRIPT $RE{param}* \s* |
    APPLET $RE{param}* \s* > [^<]* (?:<(?!/APPLET)[^<]*)* </APPLET $RE{param}* \s* |
    OBJECT $RE{param}* \s* > [^<]* (?:<(?!/OBJECT)[^<]*)* </OBJECT $RE{param}* \s*
    ) #xoi;

#    <>
my $comment = qr/ (?:
    !-- [^>]* (?: (?:(?<!--)>[^>]*)* (?<=--) )?  |
    \?  [^>]*     (?:(?<!\?)>[^>]*)* (?<=\?)
    ) /x;

#      qr//: ``Sequence (?k...) not recognized in regex''
s/\((?=\s)/(?k:/g for $tag, $ctag, $comment;
pattern name => [ 'tag'	    ], create => $tag;
pattern name => [ 'ctag'    ], create => $ctag;
pattern name => [ 'comment' ], create => $comment;

#    <>
my $anytag  = qr/ (?: $RE{ctag} | $RE{tag} | $RE{comment} ) /xo;

# (!)    &nbsp;
my $s = qr/ (?: (?:\s|&nbsp;)* (?!\s|&nbsp;) ) /x;

#      qr//: ``Sequence (?k...) not recognized in regex''
s/\((?=\s)/(?k:/g for $anytag, $s;
pattern name => [ 'anytag'  ], create => $anytag;
pattern name => [ 's'	    ], create => $s;

#     .
pattern name	=> [ '>' ],
	create	=> $RE{param}.'*\s*>'.$RE{s};

#   .
#   -not  () ().
# :   \R[<>]   \R[<>,-not=>tr]   \R[<>,-not=>'tr|/table']
pattern name	=> [ '<>' ],
	create	=> sub {
            my $not = $_[1]{'-not'} ? '(?i:(?!'.$_[1]{'-not'}.'))' : '';
            '(?:<'.$not.$RE{anytag}.'>'.$RE{s}.')';
	};

#   A    HREF.
# $1 = url ( -keep)
pattern name	=> [ 'a' ],
	create	=> '(?:<(?i:A)'.$RE{param}.'*\s+(?i:HREF)='.$RE{pval}{-keep}.
		   $RE{param}.'*\s*>'.$RE{s}.')';

#         .
# $1 =  ( -keep)
pattern name	=> [ 'NB' ],
	create	=> '(?:'.$RE{s}.'([^<]+?)'.$RE{s}.'(?=<))';

1;
